--[[
************************************************************************

beql.lua

File date: 2019-12-04T19:45:04Z
File revision: 
Project revision: 294
Project version: r294


Author: Angelike, Bayi

************************************************************************
]]--


local AddonKey = ...;

local beql = LibStub("AceAddon-3.0"):NewAddon("beql", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0");
beql.callbacks = beql.callbacks or LibStub("CallbackHandler-1.0"):New(beql, 'On', 'Off');
local L = LibStub("AceLocale-3.0"):GetLocale("beql");

local GetQuestLogTitle = _G.GetQuestLogTitle;

do
    beql.frames = {};

	beql.revision = GetAddOnMetadata(AddonKey, "Version")
	beql.revision2 = GetAddOnMetadata(AddonKey, "294")
	beql.versionstring = GetAddOnMetadata(AddonKey, "Version")

    -- @TODO: remove
	beql.questtimers = {};
	beql.PlayerName = UnitName("player");
	beql.PlayerRealm = GetRealmName();
	beql.PlayerFraction = UnitFactionGroup("player");
	beql.PlayerKey = beql.PlayerRealm .."_" .. beql.PlayerFraction .."_" .. beql.PlayerName;
end

function UIFrameCache:Reset()
    while(#self.usedFrames > 0) do
        self.usedFrames[1]:Hide();
        self:ReleaseFrame(self.usedFrames[1]);
    end
end

-- Debug Function
-- #NODOC
function beql:debugprint(...)
	if (beql.db and beql.db.profile.debug) then
		DEFAULT_CHAT_FRAME:AddMessage("bEQL: " .. table.concat({...}, ' '))
	end
end
--local print = beql.debugprint

-- toggel addonstatus 
function beql:ToggleActive()
	if (beql:IsEnabled()) then
		beql:Disable()
	else
		beql:Enable()
	end
end

-- Init beql addon
function beql:OnInitialize()

	beql:InitDB()

	beql:debugprint("Init beql Addon")

	-- create Linecache
	beql.frames.watchLineCache = UIFrameCache:New("FRAME", tostring(beql), beql.frames.questWatchFrame, "beqlWatchFrameLineTemplate");

	-- now set some scipts to our frames
    beql:initQuestlog();
    beql:initBeqlQuestlog();
    beql:initChat();
	beql:Tooltip_InitTooltip()

	beql:AddFrameScripts();
	beql:initQuestTracker();

	-- use a functioncall instand of a copy of the function to prevent hook issue
	--QuestLogListScrollFrame.update = function(...) QuestLog_Update(...) end
	--QuestLogDetailScrollChildFrame:SetScript("OnShow", function(...) QuestLogDetailScrollChildFrame_OnShow(...) end)
	--QuestLogFrame:SetScript("OnShow", function(...) QuestLog_OnShow(...) end)
	
	beql.db.RegisterCallback(beql, "OnProfileChanged", "RefreshConfig")
	beql.db.RegisterCallback(beql, "OnProfileCopied", "RefreshConfig")
	beql.db.RegisterCallback(beql, "OnProfileReset", "RefreshConfig")

	
	beql:BlizzardSetupOptions();
	beql:RegisterChatCommand("beql", function() beql:OpenBlizzardOption("main") end)

	beql:SetEnabledState(beql.db.profile.AddonStatus)

	beql:debugprint("Init beql Addon - done")
end

-- enable beql addon
function beql:OnEnable()
	beql:debugprint("Enable bEQL - START")
	beql.db.profile.AddonStatus = true;
	
	-- Check for other addons
	beql:Compatibility()

	-- Init Questlog
	beql:questLog_OnToggle()

	-- Load Questtracker
	beql:QuestTracker_ToggleStatus(beql.db.profile.disabledtracker)

	beql.db.profile.revision = beql.revision

    beql:RawHook(_G, 'IsQuestWatched', function(index) return beql:IsQuestWatched(index); end, true);

	beql:debugprint("Enable bEQL - end")
end

-- disable beql addon
function beql:OnDisable()
	beql.db.profile.AddonStatus = false;
	beql:debugprint("Disable bEQL")
	-- disable Questlog
	beql:questLog_OnToggle();
	beql.frames.questLogAnchor.UpdateFrame = nil;

	-- disable Tracker
	beql:QuestTracker_ToggleStatus(true)

	beql:UnhookAll();

	-- Update Frames
	QuestLog_Update()
end

function beql:questLogAnchor_OnEvent(event, ...)
    if (event == 'QUEST_WATCH_UPDATE') then
        beql:autoWatchQuestProgress(...);
    end
end


-- Internal for setting default values for the variables
-- #NODOC
function beql:InitDB()
	--local charfraction = beql.PlayerFraction .. "_" .. beql.PlayerName
	local defaults = {
        profile = {
            AddonStatus = true,
            Language = "SystemDefault",
            debug = false,


            -- QuestTracker
            QuestTracker = {
                Colors = {
                    Bg = CreateColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.r),
                    BgBorder = CreateColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.r),
                    Zone = CreateColor(1, 1, 1, 1),
                    HeaderNormal = CreateColor(0.75, 0.61, 0),
                    HeaderComplete = CreateColor(0.05, 0.93, 0.15),
                    HeaderFailed = CreateColor(1, 0.09, 0),
                    ObjectiveNormal = CreateColor(1, 0.09, 0),
                    ObjectiveComplete = CreateColor(0.05, 0.93, 0.15),
                    ObjetiveStepColor = CreateColor(0.96, 1, 0)
                },

                -- Color event
                UseBgColors = true,
                CustomBgColor = false,
                CustomHeaderColor = true,
                HeaderQuestLevelColor = true,
                CustomZoneColor = false,
                CustomObjetiveColor = true,
                FadeColor = true,
                Alpha = 1,

                locked = true,
                disabled = false,
                ShowHeader = true,
                FontHeight = 10,
                ShowZones = true,
                RemoveFinished = true,
                MinimizeFinished = true,
                HideCompletedOnly = false,
                ShowObjectiveMarkers = true,
                UseTrackerListing = false,
                TrackerList = 1,
                TrackerSymbol = 1,
                SortItems = true,
                AddNew = true,
                AddUntracked = true,
                AutoResize = true,
                FixedWidth = 250,
                LeftClick = true,
                RightClick = true,
                activetracker = true,
                activetrackerparty = false
            },

            -- QuestLog
            QuestLog = {
                chat = {
                    AnnounceQuest = false,
                    chattype = "default",
                    accept = false,
                    abandoned = false,
                    progress = true,
                    finished = true,
                    delivert = false
                },
                locked = true,
                showlevel = true,
                showTags = true,
                InfoOnQuestCompletion = true,
                autocomplete = false,
                style = 'extended',
                Alpha = 1,
                Scale = 1,
                InfoSound = "Sound\\Interface\\PVPFlagCapturedmono.wav",
                FontSize = 12
            },
            -- Tooltip
            Tooltip = {
                Colors = {
                    QuestTracker_TooltipTitle = CreateColor(0.75, 0.61, 0),
                    QuestTracker_TooltipDesc = CreateColor(1.0, 0.8, 0),
                    GameTooltip_MobTitle = CreateColor(1, 1, 1),
                    GameTooltip_ItemTitle = CreateColor(1, 1, 1),
                    GameTooltip_ObjectiveIncomplete = CreateColor(1, 0.09, 0),
                    GameTooltip_ObjectiveComplete = CreateColor(0.05, 0.93, 0.15),
                    GameTooltip_ObjectiveStep = CreateColor(0.96, 1, 0),
                    GameTooltip_NonObjective = CreateColor(1, 0.64, 0.03)
                },
                -- Gametooltip
                mob = true,
                item = true,
                GameTooltip_TitelIsQuestcolor = true,
                GameTooltip_ColorObjectives = true,
                GameTooltip_FadeObjectiveColors = true,
                GameTooltip_ObjectiveUseStep = true,

                -- dont know
                TooltipTitleDiff = true,
                TooltipObjFade = true,
                TooltipItemFade = true,

                -- QuestTracker 
                QuestTracker_TooltipShow = true,
                QuestTracker_TooltipTitleQuestcolor = true,
                QuestTracker_TooltipDesc = true
            },
            -- Fubar
            fubar = {
                hide = false,
                ShowTooltip = true,
                minimapPos = 238
            },

            disabled = {}
        },

        char = {
            FramePositions = {},
            QuestLogMinimized = false,
            QuestWatchMinimized = false,
            QuestWatchIDs = {},
            questlogids = {}
        },
        global = {
            questtimes = {}
        }
	}
	beql.db = LibStub("AceDB-3.0"):New("beqlDBnew", defaults, true)
end



--[[----------------------------------------
-- Arguments:
-- table - Color1
-- table - Color2
-- number - Have
-- number - Needed
--
-- Notes:
-- Fades Colors between Color1 and Color2 depending on how much percent has Have from Needed
--
-- Returns:
-- * table - Color Faded
------------------------------------------]]
function beql:FadeColors(tempColor, tempColor2, got, need)
	local color = CreateColor(0, 0, 0, 1);
	local multiplier = got / need;

	color.r = tempColor.r + ((tempColor2.r - tempColor.r) * multiplier);
	color.g = tempColor.g + ((tempColor2.g - tempColor.g) * multiplier);
	color.b = tempColor.b + ((tempColor2.b - tempColor.b) * multiplier);

	return color
end

--[[----------------------------------------
-- Arguments:
-- number - Decimal nr
-- number - Length of number
--
-- Notes:
-- Converts a Decimal Number to Hexadecimal
--
-- Returns:
-- string - Hexadecimal number
------------------------------------------]]
function beql:decToHex(Dec, Length)
	local B, K, Hex, I, D = 16, "0123456789ABCDEF", "", 0;

	while Dec>0 do
		I=I+1;
		Dec, D = math.floor(Dec/B), math.fmod(Dec, B)+1;
		Hex=string.sub(K, D, D)..Hex;
	end

	if( (Length ~= nil) and (string.len(Hex) < Length) ) then
		local temp, i = Length-string.len(Hex), 1;
		for i=1, temp, 1 do
			Hex = "0"..Hex;
		end
	end

	return Hex;
end

--[[----------------------------------------
-- Arguments:
-- string - text
-- number - Red Component
-- number - Green Component
-- number - Blue Component
--
-- Notes:
-- Returns a WoW-style colored text
--
-- Returns:
-- string - Text colored with WoW control characters
------------------------------------------]]
function beql:ColorText(t, r, g, b)
	if (t == nil) then
        t = ""; end
	if (r == nil) then
        r = 0.0;
    end
	if (g == nil) then
        g = 0.0;
    end
	if (b == nil) then
        b = 0.0;
    end
	return "|CFF"..beql:decToHex(r*255, 2)..beql:decToHex(g*255, 2)..beql:decToHex(b*255, 2)..t.."|r";
end

--[[---------------------------------------
-- Arguments:
-- string - text
--
-- Notes:
-- Prints all Points of a Region
--
-- Returns:
-- nothing

-- function to ge points of an region
-- #NODOC
-----------------------------------------]]
function beql:printpoints(tmp)
	local region
	if (type(tmp) == 'string') then
		region = getglobal(tmp)
	else
		region = tmp
	end
	local i

	print("Framepoints for " .. tostring(region:GetName()))
	for i=1, region:GetNumPoints(), 1 do
		local point, relativeTo, relativePoint, xOfs, yOfs = region:GetPoint(i)
		beql:debugprint(point .. " " .. relativeTo:GetName() .. " " .. relativePoint .. " " ..  xOfs .." ".. yOfs)
	end
end

function beql:round(input, places)
	if (not places) then
        places = 0
	end
	if (type(input) == "number" and type(places) == "number") then
        local pow = 1
        for i = 1, ceil(places) do
            pow = pow * 10
        end
        return floor(input * pow + 0.5) / pow
	end
end

function beql:RefreshConfig()
	beql:Disable()
	
	beql.frames.questLogAnchor.UpdateFrame = nil
	beql.frames.questWatchAnchor.UpdateFrame = nil
	
	if (beql.db.profile.AddonStatus) then
		beql:Enable()
	end
end

--[[---------------------------------------
-- Arguments:
-- none
--
-- Notes:
-- Adds scripts to our Fremes, which where created in frames.xml
--
-- Returns:
-- nothing

-- #NODOC
-----------------------------------------]]
function beql:AddFrameScripts()
	-- OnUpdate functions
	
	-- Anchor Frame for Questlog
	beql.frames.questLogAnchor:SetScript('OnUpdate', function(self, elapsed)
		if (self.UIScale ~= UIParent:GetScale()) then
			beql:RestoreFramePosition(self)
		end
        self:SetScript('OnUpdate', nil);
	end)
	
	
	-- ToDo Watchframline virtual template script dispatcher (OnUpdate)
	
	-- QuestLog Config Button
	beql.frames.configButton:SetScript('OnClick', function(self, button, pushed)
		beql:OpenBlizzardOption("main");
	end)
	
	-- QuestLog Ninimize Button
	beql.frames.questLogToggleButton:SetScript('OnClick', function(self, button, pushed)
		if (beql.db.char.QuestLogMinimized) then
			beql:QuestLog_Maximize(self)
		else
			beql:QuestLog_Minimize(self)
		end
	end)

	
	-- OnDrag Functions
	-- QuestLogFrame
	QuestLogFrame:SetScript("OnDragStart", function(self)
		if (beql.frames.questLogAnchor:IsMovable()) then
			beql.frames.questLogAnchor.ismoving = true;
			beql.frames.questLogAnchor:StartMoving();
		end
	end)
	QuestLogFrame:SetScript("OnDragStop", function(self)
		if (beql.frames.questLogAnchor:IsMovable()) then
			beql.frames.questLogAnchor:StopMovingOrSizing();
			beql.frames.questLogAnchor.ismoving = false;
			beql:SaveFramePosition(beql.frames.questLogAnchor);
		end
	end)
end


--[[---------------------------------------
-- Arguments:
-- string - name ot the frame
-- UIObject - reference of the frame
--
-- Notes:
-- Adds Frames to our addon, which where created in frames.xml
--
-- Returns:
-- nothing

-- #NODOC
-----------------------------------------]]
function beql:setFrame(name, frame)
    beql.frames[name] = frame;
end

function lcfirst(string)
    if (type(string) == 'string') then
        return string:gsub('^.', strlower(string:match('^.')))
    end

    return string;
end


-- EOF ---
